from ctypes import *
import wx
import braille

try:
	mbDll=windll.LoadLibrary("brailleDisplayDrivers\\mb408sl.dll")
except:
	mbDll=None

mbCellsMap=[]
KEY_CHECK_INTERVAL = 50

def convertMbCells(cell):
	newCell = (1<<6 if cell & 1<<4 else 0) | \
		(1<<5 if cell & 1<<5  else 0) | \
		(1<<0 if cell & 1<<6  else 0) | \
		(1<<3 if cell & 1<<0  else 0) | \
		(1<<2 if cell & 1<<1  else 0) | \
		(1<<1 if cell & 1<<2  else 0) | \
		(1<<7 if cell & 1<<3  else 0) | \
		(1<<4 if cell & 1<<7  else 0) 
	return newCell

class BrailleDisplayDriver(braille.BrailleDisplayDriverWithCursor):
	name = "mb408sl"
	description = _("MB408SL")

	@classmethod
	def check(cls):
		return bool(mbDll)


	def  __init__(self):
		global mbCellsMap
		super(BrailleDisplayDriver, self).__init__()
		mbCellsMap=[convertMbCells(x) for x in range(256)]
		if (mbDll.BrlInit("COM1", 19200)):
			self._keyCheckTimer = wx.PyTimer(self._handleKeyPresses)
			self._keyCheckTimer.Start(KEY_CHECK_INTERVAL)
		else:
			raise RuntimeError("No display found")


	def terminate(self):
		super(BrailleDisplayDriver, self).terminate()
		try:
			self._keyCheckTimer.Stop()
			self._keyCheckTimer = None
		except:
			pass
		mbDll.BrlTerminate()

	def _get_numCells(self):
		return 40

	def _handleKeyPresses(self):
		while True:
			try:
				key=mbDll.ReadBuf()
			except:
				pass
			if not key: break
			if key==11: braille.handler.scrollBack()
			elif key==13: braille.handler.scrollForward()
			elif (key >= 257) and (key <= 296): braille.handler.routeTo(key-257) 

	def _display(self, cells):
		cells="".join(chr(mbCellsMap[x]) for x in cells)+"".join(chr(mbCellsMap[0]) for x in range(40-len(cells)))
		mbDll.WriteBuf(cells) 
		self._handleKeyPresses()
